@@ -4,13 +4,14 @@ Line-based transformers are the simpler ones; token-based transformers are
 more complex. See test_inputtransformer2_line for tests for line-based
 transformations.
 """
-import nose.tools as nt
 import string
+import sys
+from textwrap import dedent
+
+import pytest
 
 from IPython.core import inputtransformer2 as ipt2
-from IPython.core.inputtransformer2 import make_tokens_by_line, _find_assign_op
-
-from textwrap import dedent
+from IPython.core.inputtransformer2 import _find_assign_op, make_tokens_by_line
 
 MULTILINE_MAGIC = ("""\
 a = f()
@@ -31,6 +32,16 @@ for a in range(5):
     get_ipython().run_line_magic('ls', '')
 """.splitlines(keepends=True))
 
+CRLF_MAGIC = ([
+    "a = f()\n",
+    "%ls\r\n",
+    "g()\n"
+], (2, 0), [
+    "a = f()\n",
+    "get_ipython().run_line_magic('ls', '')\n",
+    "g()\n"
+])
+
 MULTILINE_MAGIC_ASSIGN = ("""\
 a = f()
 b = %foo \\
@@ -131,15 +142,17 @@ def null_cleanup_transformer(lines):
     """
     return []
 
-def check_make_token_by_line_never_ends_empty():
+
+def test_check_make_token_by_line_never_ends_empty():
     """
     Check that not sequence of single or double characters ends up leading to en empty list of tokens
     """
     from string import printable
     for c in printable:
-        nt.assert_not_equal(make_tokens_by_line(c)[-1], [])
+        assert make_tokens_by_line(c)[-1] != []
         for k in printable:
-            nt.assert_not_equal(make_tokens_by_line(c+k)[-1], [])
+            assert make_tokens_by_line(c + k)[-1] != []
+
 
 def check_find(transformer, case, match=True):
     sample, expected_start, _  = case
@@ -147,21 +160,21 @@ def check_find(transformer, case, match=True):
     res = transformer.find(tbl)
     if match:
         # start_line is stored 0-indexed, expected values are 1-indexed
-        nt.assert_equal((res.start_line+1, res.start_col), expected_start)
+        assert (res.start_line + 1, res.start_col) == expected_start
         return res
     else:
-        nt.assert_is(res, None)
+        assert res is None
 
 def check_transform(transformer_cls, case):
     lines, start, expected = case
     transformer = transformer_cls(start)
-    nt.assert_equal(transformer.transform(lines), expected)
+    assert transformer.transform(lines) == expected
 
 def test_continued_line():
     lines = MULTILINE_MAGIC_ASSIGN[0]
-    nt.assert_equal(ipt2.find_end_of_continued_line(lines, 1), 2)
+    assert ipt2.find_end_of_continued_line(lines, 1) == 2
 
-    nt.assert_equal(ipt2.assemble_continued_line(lines, (1, 5), 2), "foo    bar")
+    assert ipt2.assemble_continued_line(lines, (1, 5), 2) == "foo    bar"
 
 def test_find_assign_magic():
     check_find(ipt2.MagicAssign, MULTILINE_MAGIC_ASSIGN)
@@ -190,6 +203,7 @@ def test_find_magic_escape():
 def test_transform_magic_escape():
     check_transform(ipt2.EscapedCommand, MULTILINE_MAGIC)
     check_transform(ipt2.EscapedCommand, INDENTED_MAGIC)
+    check_transform(ipt2.EscapedCommand, CRLF_MAGIC)
 
 def test_find_autocalls():
     for case in [AUTOCALL_QUOTE, AUTOCALL_QUOTE2, AUTOCALL_PAREN]:
@@ -206,12 +220,12 @@ def test_find_help():
         check_find(ipt2.HelpEnd, case)
 
     tf = check_find(ipt2.HelpEnd, HELP_CONTINUED_LINE)
-    nt.assert_equal(tf.q_line, 1)
-    nt.assert_equal(tf.q_col, 3)
+    assert tf.q_line == 1
+    assert tf.q_col == 3
 
     tf = check_find(ipt2.HelpEnd, HELP_MULTILINE)
-    nt.assert_equal(tf.q_line, 1)
-    nt.assert_equal(tf.q_col, 8)
+    assert tf.q_line == 1
+    assert tf.q_col == 8
 
     # ? in a comment does not trigger help
     check_find(ipt2.HelpEnd, (["foo # bar?\n"], None, None), match=False)
@@ -220,16 +234,16 @@ def test_find_help():
 
 def test_transform_help():
     tf = ipt2.HelpEnd((1, 0), (1, 9))
-    nt.assert_equal(tf.transform(HELP_IN_EXPR[0]), HELP_IN_EXPR[2])
+    assert tf.transform(HELP_IN_EXPR[0]) == HELP_IN_EXPR[2]
 
     tf = ipt2.HelpEnd((1, 0), (2, 3))
-    nt.assert_equal(tf.transform(HELP_CONTINUED_LINE[0]), HELP_CONTINUED_LINE[2])
+    assert tf.transform(HELP_CONTINUED_LINE[0]) == HELP_CONTINUED_LINE[2]
 
     tf = ipt2.HelpEnd((1, 0), (2, 8))
-    nt.assert_equal(tf.transform(HELP_MULTILINE[0]), HELP_MULTILINE[2])
+    assert tf.transform(HELP_MULTILINE[0]) == HELP_MULTILINE[2]
 
     tf = ipt2.HelpEnd((1, 0), (1, 0))
-    nt.assert_equal(tf.transform(HELP_UNICODE[0]), HELP_UNICODE[2])
+    assert tf.transform(HELP_UNICODE[0]) == HELP_UNICODE[2]
 
 def test_find_assign_op_dedent():
     """
@@ -239,31 +253,60 @@ def test_find_assign_op_dedent():
         def __init__(self, s):
             self.string = s
 
-    nt.assert_equal(_find_assign_op([Tk(s) for s in ('','a','=','b')]), 2)
-    nt.assert_equal(_find_assign_op([Tk(s) for s in ('','(', 'a','=','b', ')', '=' ,'5')]), 6)
+    assert _find_assign_op([Tk(s) for s in ("", "a", "=", "b")]) == 2
+    assert (
+        _find_assign_op([Tk(s) for s in ("", "(", "a", "=", "b", ")", "=", "5")]) == 6
+    )
 
+
+examples = [
+    pytest.param("a = 1", "complete", None),
+    pytest.param("for a in range(5):", "incomplete", 4),
+    pytest.param("for a in range(5):\n    if a > 0:", "incomplete", 8),
+    pytest.param("raise = 2", "invalid", None),
+    pytest.param("a = [1,\n2,", "incomplete", 0),
+    pytest.param("(\n))", "incomplete", 0),
+    pytest.param("\\\r\n", "incomplete", 0),
+    pytest.param("a = '''\n   hi", "incomplete", 3),
+    pytest.param("def a():\n x=1\n global x", "invalid", None),
+    pytest.param(
+        "a \\ ",
+        "invalid",
+        None,
+        marks=pytest.mark.xfail(
+            reason="Bug in python 3.9.8 – bpo 45738",
+            condition=sys.version_info
+            in [(3, 9, 8, "final", 0), (3, 11, 0, "alpha", 2)],
+            raises=SystemError,
+            strict=True,
+        ),
+    ),  # Nothing allowed after backslash,
+    pytest.param("1\\\n+2", "complete", None),
+]
+
+
+@pytest.mark.parametrize("code, expected, number", examples)
+def test_check_complete_param(code, expected, number):
+    cc = ipt2.TransformerManager().check_complete
+    assert cc(code) == (expected, number)
+
+
+@pytest.mark.xfail(
+    reason="Bug in python 3.9.8 – bpo 45738",
+    condition=sys.version_info in [(3, 9, 8, "final", 0), (3, 11, 0, "alpha", 2)],
+    raises=SystemError,
+    strict=True,
+)
 def test_check_complete():
     cc = ipt2.TransformerManager().check_complete
-    nt.assert_equal(cc("a = 1"), ('complete', None))
-    nt.assert_equal(cc("for a in range(5):"), ('incomplete', 4))
-    nt.assert_equal(cc("for a in range(5):\n    if a > 0:"), ('incomplete', 8))
-    nt.assert_equal(cc("raise = 2"), ('invalid', None))
-    nt.assert_equal(cc("a = [1,\n2,"), ('incomplete', 0))
-    nt.assert_equal(cc(")"), ('incomplete', 0))
-    nt.assert_equal(cc("\\\r\n"), ('incomplete', 0))
-    nt.assert_equal(cc("a = '''\n   hi"), ('incomplete', 3))
-    nt.assert_equal(cc("def a():\n x=1\n global x"), ('invalid', None))
-    nt.assert_equal(cc("a \\ "), ('invalid', None))  # Nothing allowed after backslash
-    nt.assert_equal(cc("1\\\n+2"), ('complete', None))
-    nt.assert_equal(cc("exit"), ('complete', None))
 
     example = dedent("""
         if True:
             a=1""" )
 
-    nt.assert_equal(cc(example), ('incomplete', 4))
-    nt.assert_equal(cc(example+'\n'), ('complete', None))
-    nt.assert_equal(cc(example+'\n    '), ('complete', None))
+    assert cc(example) == ("incomplete", 4)
+    assert cc(example + "\n") == ("complete", None)
+    assert cc(example + "\n    ") == ("complete", None)
 
     # no need to loop on all the letters/numbers.
     short = '12abAB'+string.printable[62:]
@@ -273,7 +316,8 @@ def test_check_complete():
         for k in short:
             cc(c+k)
 
-    nt.assert_equal(cc("def f():\n  x=0\n  \\\n  "), ('incomplete', 2))
+    assert cc("def f():\n  x=0\n  \\\n  ") == ("incomplete", 2)
+
 
 def test_check_complete_II():
     """
@@ -283,13 +327,31 @@ def test_check_complete_II():
 
     """
     cc = ipt2.TransformerManager().check_complete
-    nt.assert_equal(cc('''def foo():\n    """'''), ('incomplete', 4))
+    assert cc('''def foo():\n    """''') == ("incomplete", 4)
+
+
+def test_check_complete_invalidates_sunken_brackets():
+    """
+    Test that a single line with more closing brackets than the opening ones is
+    interpreted as invalid
+    """
+    cc = ipt2.TransformerManager().check_complete
+    assert cc(")") == ("invalid", None)
+    assert cc("]") == ("invalid", None)
+    assert cc("}") == ("invalid", None)
+    assert cc(")(") == ("invalid", None)
+    assert cc("][") == ("invalid", None)
+    assert cc("}{") == ("invalid", None)
+    assert cc("]()(") == ("invalid", None)
+    assert cc("())(") == ("invalid", None)
+    assert cc(")[](") == ("invalid", None)
+    assert cc("()](") == ("invalid", None)
 
 
 def test_null_cleanup_transformer():
     manager = ipt2.TransformerManager()
     manager.cleanup_transforms.insert(0, null_cleanup_transformer)
-    assert  manager.transform_cell("") == ""
+    assert manager.transform_cell("") == ""
 
 
 
