@@ -2,18 +2,14 @@
 """Test IPython.core.logger"""
 
 import os.path
+import pytest
 
-import nose.tools as nt
 from IPython.utils.tempdir import TemporaryDirectory
 
 def test_logstart_inaccessible_file():
-    try:
+    with pytest.raises(IOError):
         _ip.logger.logstart(logfname="/")   # Opening that filename will fail.
-    except IOError:
-        pass
-    else:
-        nt.assert_true(False)           # The try block should never pass.
-    
+
     try:
         _ip.run_cell("a=1")                 # Check it doesn't try to log this
     finally:
